from django.core.exceptions import ObjectDoesNotExist
from tqdm import tqdm

from meta_consent.models import SubjectConsent
from meta_rando.models import RandomizationList

from ..models import Label, LotNumber, Rx


class InvalidLotNumber(Exception):
    pass


def get_label_data(lot_no, max_labels: int | None = None) -> list[Label]:
    try:
        lot_no = LotNumber.objects.get(lot_no=lot_no)
    except ObjectDoesNotExist:
        raise ObjectDoesNotExist("The lot number given is invalid")
    else:
        labels = []
        qs = SubjectConsent.objects.values("subject_identifier", "site_id", "gender").all()
        for obj in tqdm(qs, total=qs.count()):
            if max_labels and len(labels) >= max_labels:
                break
            rando = RandomizationList.objects.get(subject_identifier=obj["subject_identifier"])
            if rando.assignment != lot_no.allocation:
                continue
            rx = Rx.objects.get(subject_identifier=obj["subject_identifier"])
            labels.append(Label.objects.create(rx=rx, lot_no=lot_no))
    return labels



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import meta_pharmacy_admin
from ..models import LotNumber


@admin.register(LotNumber, site=meta_pharmacy_admin)
class LotNumberAdmin(ModelAdminSubjectDashboardMixin, admin.ModelAdmin):
    """Admin class for proxy model of edc_pharmacy.Rx"""

    show_object_tools = True

    # form = LotNumberForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "lot_no",
                    "expiration_date",
                    "allocation",
                    "qty",
                )
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "lot_no",
        "expiration_date",
    )

    radio_fields = {"allocation": admin.VERTICAL}

    list_filter = ("allocation",)

    search_fields = [
        "lot_no",
    ]



from django.db import models
from edc_model.models import BaseUuidModel
from edc_pharmacy.models import Medication
from edc_randomization.constants import ACTIVE, PLACEBO


class LotNumber(BaseUuidModel):

    lot_no = models.CharField(max_length=25, unique=True)

    medication = models.ForeignKey(
        Medication, on_delete=models.PROTECT, null=True, blank=False
    )

    allocation = models.CharField(
        max_length=25, choices=((ACTIVE, "Active"), (PLACEBO, "Placebo"))
    )

    expiration_date = models.DateField()

    qty = models.IntegerField(null=True)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Lot Number"
        verbose_name_plural = "Lot Numbers"



from django.db import models
from edc_identifier.simple_identifier import SimpleUniqueIdentifier
from edc_model.models import BaseUuidModel

from .lot_number import LotNumber
from .rx import Rx


class LabelIdentifier(SimpleUniqueIdentifier):
    random_string_length = 6
    identifier_type = "rx_label_reference"
    template = "{random_string}"


class Label(BaseUuidModel):

    identifier_cls = LabelIdentifier
    identifier_field_name: str = "rx_label_reference"

    rx = models.ForeignKey(Rx, on_delete=models.PROTECT)
    lot_no = models.ForeignKey(LotNumber, on_delete=models.PROTECT)
    rx_label_reference = models.CharField(max_length=15, unique=True)

    printed_datetime = models.DateTimeField(null=True)
    printed = models.BooleanField(default=False)
    scanned = models.BooleanField(default=False)
    scanned_datetime = models.DateTimeField(null=True)

    def __str__(self):
        return self.rx_label_reference

    def save(self, *args, **kwargs):
        """Label referenceis always allocated."""
        if not self.id:
            setattr(
                self,
                self.identifier_field_name,
                self.identifier_cls().identifier,
            )
        super().save(*args, **kwargs)  # type:ignore

    @property
    def human_readable_identifier(self):
        """Returns a humanized screening identifier."""
        x = self.screening_identifier
        return f"{x[0:4]}-{x[4:]}"

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Label"
        verbose_name_plural = "Labels"
